unit NeginSMS;

interface

uses SysUtils, AlmasSMS, SOAPHTTPClient;

type
  TArrayOfInt64 = array of Int64;
  TSMS = record
    Body, PhoneNumber, MessageID: WideString;
    EnteredDate: TDateTime;
  end;
  TArrayOfSMS = array of TSMS;

  procedure SMSInit(const Username, Password: WideString; UseLocalIntranet: Boolean = False);
  function GetCredits: Int64;
  function SendSMS(Mobiles, Bodys, UDHs: ArrayOfString): TArrayOfInt64;
  function GetSMS(var SMSArray: TArrayOfSMS): Boolean;

implementation

var
	FSoapSms: AlmasSmsSoap;
  FUserName, FPassword: WideString;

function SoapSms: AlmasSmsSoap;
begin
  if FSoapSms = nil then
    raise Exception.Create('SMS functions are not initialized. Use SMSInit() first');
  Result:= FSoapSms;
end;

procedure SMSInit(const Username, Password: WideString; UseLocalIntranet: Boolean = False);
var
  HTTPRIO: THTTPRIO;
begin
  HTTPRIO:= THTTPRIO.Create(nil);
  HTTPRIO.Converter.Encoding:= 'UTF8';
  HTTPRIO.HTTPWebNode.UseUTF8InHeader:= True;
  
  if UseLocalIntranet then
    FSoapSms := GetAlmasSmsSoap(True, 'http://lws.3300.ir/AlmasSms.asmx?wsdl', HTTPRIO)
  else
    FSoapSms := GetAlmasSmsSoap(True, 'http://iws.3300.ir/AlmasSms.asmx?wsdl', HTTPRIO);
  FUserName:= Username;
  FPassword:= Password;
end;

function GetCredits: Int64;
var
  Params: GetCredit;
  Response: GetCreditResponse;
begin
  Result:= -1;
  Params:= GetCredit.Create;
  Response:= GetCreditResponse.Create;
  try
    Params.pUsername:= FUserName;
    Params.pPassword:= FPassword;
    with SoapSms.GetCredit(Params) do
    begin
      if GetCreditResult >= 0 then
        raise Exception.CreateFmt('Error in get credit code: %d', [GetCreditResult]);
      Result:= pCredit;
    end;
  finally
    Params.Free;
    Response.Free;
  end;
end;

function SendSMS(Mobiles, Bodys, UDHs: ArrayOfString): TArrayOfInt64;
var
  Encoding, MClass: ArrayOfInt;
  I, Cnt: Integer;
  Res: Int64;
  Response: SendSms2Response;
  a: SendSms2;
begin
  a:= SendSms2.Create;
  try
    Cnt:= High(Mobiles) + 1;
    SetLength(Mobiles, Cnt);
    SetLength(Encoding, Cnt);
    SetLength(MClass, Cnt);

    for I := 0 to Cnt - 1 do
    begin
      Encoding[I]:= 0;
      MClass[I]:= 1;
    end;

    a.pUsername:= FUserName;
    a.pPassword:= FPassword;
    a.messages:= Bodys;
    a.mobiles:= Mobiles;
    a.UDH:= UDHs;
    a.encodings:= Encoding;
    a.mclass:= MClass;
    Response:= SoapSms.SendSms2(a);
    begin
    Res:= Response.SendSms2Result;
    if Res >= 0 then
      raise Exception.CreateFmt('Error in Send SMS code: %d', [Res]);
    SetLength(Result, Cnt);
    for I:= 0 to Cnt - 1 do
      Result[I]:= Response.pMessageIds[I];
    end;
    a.Free;
  except
    on E: Exception do
    begin
    a.Free;
    raise Exception.Create(E.Message);
    end;
  end;
end;

function GetSMS(var SMSArray: TArrayOfSMS): Boolean;
var
  AGetReceivedSMS: GetReceivedSms2;
  Res: GetReceivedSms2Response;
  I: Integer;
begin
  AGetReceivedSMS:= GetReceivedSMS2.Create;
  try
    AGetReceivedSMS.pUsername:= FUserName;
    AGetReceivedSMS.pPassword:= FPassword;
    Res:= SoapSms.GetReceivedSms2(AGetReceivedSms);
    begin
      Result:= (Res.GetReceivedSms2Result < 0);
      if Result then
      begin
        Result:= High(Res.pMessages) >= 0;
        SetLength(SMSArray, High(Res.pMessages) + 1);
        for I := 0 to High(Res.pMessages) do
        begin
          SMSArray[I].Body:= (Res.pMessages[I]);
          SMSArray[I].PhoneNumber:= Res.pFroms[I];
          SMSArray[I].MessageID:= Res.pMessageIds[I];
          SMSArray[I].EnteredDate:= Res.pEnteredDate[I].AsDateTime;
        end
      end
      else
        raise Exception.Create('Error in get sms code'+
         IntToStr(Res.GetReceivedSms2Result));
    end;
    AGetReceivedSMS.Free;
  except
    on E: Exception do
    begin
      AGetReceivedSMS.Free;
      raise Exception.Create(E.Message);
    end;
  end;
end;

end.
